@@ -55,6 +55,8 @@ static char *flist_dir = NULL;
 
 static void send_file_entry(struct file_struct *file,int f)
 {
+  if (f == -1) return;
+
   write_int(f,strlen(file->name));
   write_buf(f,file->name,strlen(file->name));
   write_int(f,(int)file->modtime);
@@ -147,8 +149,6 @@ static struct file_struct *make_file(int recurse,char *fname)
 }
 
 
-static int flist_malloced;
-
 static void send_file_name(int f,struct file_list *flist,
 			   int recurse,char *fname)
 {
@@ -158,11 +158,11 @@ static void send_file_name(int f,struct file_list *flist,
 
   if (!file) return;
   
-  if (flist->count >= flist_malloced) {
-    flist_malloced += 100;
+  if (flist->count >= flist->malloced) {
+    flist->malloced += 100;
     flist->files = (struct file_struct *)realloc(flist->files,
 						 sizeof(flist->files[0])*
-						 flist_malloced);
+						 flist->malloced);
     if (!flist->files)
       out_of_memory("send_file_name");
   }
@@ -228,13 +228,16 @@ struct file_list *send_file_list(int f,int recurse,int argc,char *argv[])
   char dbuf[MAXPATHLEN];
   struct file_list *flist;
 
+  if (verbose && recurse)
+    fprintf(am_server?stderr:stdout,"building file list ... ");
+
   flist = (struct file_list *)malloc(sizeof(flist[0]));
   if (!flist) out_of_memory("send_file_list");
 
   flist->count=0;
-  flist_malloced = 100;
+  flist->malloced = 100;
   flist->files = (struct file_struct *)malloc(sizeof(flist->files[0])*
-					      flist_malloced);
+					      flist->malloced);
   if (!flist->files) out_of_memory("send_file_list");
 
   for (i=0;i<argc;i++) {
@@ -290,11 +293,16 @@ struct file_list *send_file_list(int f,int recurse,int argc,char *argv[])
     send_file_name(f,flist,recurse,fname);
   }
 
-  write_int(f,0);
-  write_flush(f);
+  if (f != -1) {
+    write_int(f,0);
+    write_flush(f);
+  }
 
   clean_flist(flist);
 
+  if (verbose && recurse)
+    fprintf(am_server?stderr:stdout,"done\n");
+
   return flist;
 }
 
@@ -303,7 +311,6 @@ struct file_list *recv_file_list(int f)
 {
   int l;
   struct file_list *flist;
-  int malloc_count=0;
 
   if (verbose > 2)
     fprintf(stderr,"recv_file_list starting\n");
@@ -312,22 +319,22 @@ struct file_list *recv_file_list(int f)
   if (!flist)
     goto oom;
 
-  malloc_count=100;
+  flist->count=0;
+  flist->malloced=100;
   flist->files = (struct file_struct *)malloc(sizeof(flist->files[0])*
-					      malloc_count);
+					      flist->malloced);
   if (!flist->files)
     goto oom;
 
-  flist->count=0;
 
   for (l=read_int(f); l; l=read_int(f)) {
     int i = flist->count;
 
-    if (i >= malloc_count) {
-      malloc_count += 100;
+    if (i >= flist->malloced) {
+      flist->malloced += 100;
       flist->files =(struct file_struct *)realloc(flist->files,
 						  sizeof(flist->files[0])*
-						  malloc_count);
+						  flist->malloced);
       if (!flist->files)
 	goto oom;
     }
@@ -385,10 +392,31 @@ oom:
 
 static int flist_compare(struct file_struct *f1,struct file_struct *f2)
 {
-  if (!f1->name || !f2->name) return -1;
+  if (!f1->name && !f2->name) return 0;
+  if (!f1->name) return -1;
+  if (!f2->name) return 1;
   return strcmp(f1->name,f2->name);
 }
 
+
+int flist_find(struct file_list *flist,struct file_struct *f)
+{
+  int low=0,high=flist->count;
+
+  while (low != high) {
+    int mid = (low+high)/2;
+    int ret = flist_compare(&flist->files[mid],f);
+    if (ret == 0) return mid;
+    if (ret > 0) 
+      high=mid;
+    else
+      low=mid+1;
+  }
+  if (flist_compare(&flist->files[low],f) == 0)
+    return low;
+  return -1;
+}
+
 /*
  * This routine ensures we don't have any duplicate names in our file list.
  * duplicate names can cause corruption because of the pipelining 
