@@ -38,10 +38,12 @@ int update_only = 0;
 int cvs_exclude = 0;
 int dry_run=0;
 int local_server=0;
+int ignore_times=0;
+int delete_mode=0;
 
 int am_server = 0;
 static int sender = 0;
-static int recurse = 0;
+int recurse = 0;
 
 static void usage(FILE *f);
 
@@ -117,6 +119,8 @@ static void server_options(char **args,int *argc)
     argstr[x++] = 'c';
   if (cvs_exclude)
     argstr[x++] = 'C';
+  if (ignore_times)
+    argstr[x++] = 'I';
   argstr[x] = 0;
 
   if (x != 1) args[ac++] = argstr;
@@ -125,6 +129,9 @@ static void server_options(char **args,int *argc)
     sprintf(bsize,"-B%d",block_size);
     args[ac++] = bsize;
   }    
+  
+  if (delete_mode)
+    args[ac++] = "--delete";
 
   *argc = ac;
 }
@@ -344,6 +351,8 @@ static void usage(FILE *f)
   fprintf(f,"-B, --block-size SIZE    checksum blocking size\n");  
   fprintf(f,"-e, --rsh COMMAND        specify rsh replacement\n");
   fprintf(f,"-C, --cvs-exclude        auto ignore files in the same way CVS does\n");
+  fprintf(f,"    --delete             delete files that don't exist on the sending side\n");
+  fprintf(f,"-I, --ignore-times       don't exclude files that match length and time\n");
   fprintf(f,"    --exclude FILE       exclude file FILE\n");
   fprintf(f,"    --exclude-from FILE  exclude files listed in FILE\n");
   fprintf(f,"    --suffix SUFFIX      override backup suffix\n");  
@@ -355,16 +364,18 @@ static void usage(FILE *f)
 }
 
 enum {OPT_VERSION,OPT_SUFFIX,OPT_SENDER,OPT_SERVER,OPT_EXCLUDE,
-      OPT_EXCLUDE_FROM};
+      OPT_EXCLUDE_FROM,OPT_DELETE};
 
-static char *short_options = "oblpguDCtcahvrne:B:";
+static char *short_options = "oblpguDCtcahvrIne:B:";
 
 static struct option long_options[] = {
   {"version",     0,     0,    OPT_VERSION},
   {"server",      0,     0,    OPT_SERVER},
   {"sender",      0,     0,    OPT_SENDER},
+  {"delete",      0,     0,    OPT_DELETE},
   {"exclude",     1,     0,    OPT_EXCLUDE},
   {"exclude-from",1,     0,    OPT_EXCLUDE_FROM},
+  {"ignore-times",0,     0,    'I'},
   {"help",        0,     0,    'h'},
   {"dry-run",     0,     0,    'n'},
   {"cvs-exclude", 0,     0,    'C'},
@@ -415,6 +426,14 @@ int main(int argc,char *argv[])
 	  backup_suffix = optarg;
 	  exit(0);
 
+	case 'I':
+	  ignore_times = 1;
+	  break;
+
+	case OPT_DELETE:
+	  delete_mode = 1;
+	  break;
+
 	case OPT_EXCLUDE:
 	  add_exclude(optarg);
 	  break;
