@@ -36,6 +36,10 @@ extern int preserve_uid;
 extern int preserve_gid;
 extern int preserve_times;
 extern int dry_run;
+extern int ignore_times;
+extern int recurse;
+extern int delete_mode;
+extern int cvs_exclude;
 
 /*
   free a sums struct
@@ -224,19 +228,18 @@ static void set_perms(char *fname,struct file_struct *file,struct stat *st,
   }
 #endif
 
-#ifdef HAVE_CHOWN
   if ((preserve_uid && st->st_uid != file->uid) || 
-      (preserve_gid && st->st_gid != file->gid) {
+      (preserve_gid && st->st_gid != file->gid)) {
     updated = 1;
     if (chown(fname,
 	      preserve_uid?file->uid:-1,
 	      preserve_gid?file->gid:-1) != 0) {
-      fprintf(stderr,"chown %s : %s\n",fname,strerror(errno));
+      if (verbose>1 || preserve_uid)
+	fprintf(stderr,"chown %s : %s\n",fname,strerror(errno));
       return;
     }
   }
-#endif
-
+    
   if (verbose > 1 && report) {
     if (updated)
       fprintf(am_server?stderr:stdout,"%s\n",fname);
@@ -344,7 +347,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
   }
 
   if (st.st_size == flist->files[i].length &&
-      (st.st_mtime == flist->files[i].modtime ||
+      ((!ignore_times && st.st_mtime == flist->files[i].modtime) ||
        (always_checksum && S_ISREG(st.st_mode) && 	  
 	memcmp(sum,flist->files[i].sum,SUM_LENGTH) == 0))) {
     set_perms(fname,&flist->files[i],&st,1);
@@ -438,6 +441,29 @@ static void receive_data(int f_in,char *buf,int fd)
 }
 
 
+static void delete_files(struct file_list *flist)
+{
+  struct file_list *local_file_list;
+  char *dot=".";
+  int i;
+
+  if (!(local_file_list = send_file_list(-1,recurse,1,&dot)))
+    return;
+
+  for (i=0;i<local_file_list->count;i++) {
+    if (!local_file_list->files[i].name) continue;
+    if (S_ISDIR(local_file_list->files[i].mode)) continue;
+    if (-1 == flist_find(flist,&local_file_list->files[i])) {
+      if (verbose)
+	fprintf(stderr,"deleting %s\n",local_file_list->files[i].name);
+      if (!dry_run) {
+	if (unlink(local_file_list->files[i].name) != 0)
+	  fprintf(stderr,"unlink %s : %s\n",
+		  local_file_list->files[i].name,strerror(errno));
+      }
+    }    
+  }
+}
 
 int recv_files(int f_in,struct file_list *flist,char *local_name)
 {  
@@ -451,6 +477,12 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
   if (verbose > 2)
     fprintf(stderr,"recv_files(%d) starting\n",flist->count);
 
+  if (recurse && delete_mode && !local_name && flist->count>0) {
+    if (cvs_exclude)
+      add_cvs_excludes();
+    delete_files(flist);
+  }
+
   while (1) 
     {
       i = read_int(f_in);
@@ -491,7 +523,10 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
 
       if (st.st_size > 0) {
 	buf = map_file(fd1,st.st_size);
-	if (!buf) return -1;
+	if (!buf) {
+	  fprintf(stderr,"map_file failed\n");
+	  return -1;
+	}
       } else {
 	buf = NULL;
       }
@@ -501,10 +536,15 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
 
       /* open tmp file */
       sprintf(fnametmp,"%s.XXXXXX",fname);
-      if (NULL == mktemp(fnametmp)) 
+      if (NULL == mktemp(fnametmp)) {
+	fprintf(stderr,"mktemp %s failed\n",fnametmp);
 	return -1;
+      }
       fd2 = open(fnametmp,O_WRONLY|O_CREAT,st.st_mode);
-      if (fd2 == -1) return -1;
+      if (fd2 == -1) {
+	fprintf(stderr,"open %s : %s\n",fnametmp,strerror(errno));
+	return -1;
+      }
 
       if (!am_server && verbose)
 	printf("%s\n",fname);
@@ -582,8 +622,10 @@ off_t send_files(struct file_list *flist,int f_out,int f_in)
       }
 
       s = receive_sums(f_in);
-      if (!s) 
+      if (!s) {
+	fprintf(stderr,"receive_sums failed\n");
 	return -1;
+      }
 
       fd = open(fname,O_RDONLY);
       if (fd == -1) {
@@ -593,12 +635,17 @@ off_t send_files(struct file_list *flist,int f_out,int f_in)
       }
   
       /* map the local file */
-      if (fstat(fd,&st) != 0) 
+      if (fstat(fd,&st) != 0) {
+	fprintf(stderr,"fstat failed : %s\n",strerror(errno));
 	return -1;
+      }
       
       if (st.st_size > 0) {
 	buf = map_file(fd,st.st_size);
-	if (!buf) return -1;
+	if (!buf) {
+	  fprintf(stderr,"map_file failed : %s\n",strerror(errno));	  
+	  return -1;
+	}
       } else {
 	buf = NULL;
       }
